package com.cm6123.monopoly.app;

import com.cm6123.monopoly.game.Banker;
import com.cm6123.monopoly.game.Player;
import com.cm6123.monopoly.game.GameHandler;
import com.cm6123.monopoly.game.GameEndings;
import com.cm6123.monopoly.game.Movement;
import com.cm6123.monopoly.game.Board;
import com.cm6123.monopoly.game.GameAction;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;



public final class Application {
    /**
     * Create a logger for the class.
     */
    private static final Logger LOGGER = LoggerFactory.getLogger(Application.class);


    private Application() {
    }

    /**
     * main entry point.  If args provided, result is displayed and program exists. Otherwise, user is prompted for
     * input.
     *
     * @param args command line args.
     */
    public static void main(final String[] args) {

        LOGGER.info("Application Started with args:{}", String.join(",", args));

        System.out.println("Hello. Welcome to Monopoly.");
        Scanner scanner = new Scanner(System.in);

        System.out.println("Please enter how many rounds to play");
        int numberOfRounds = scanner.nextInt();

        System.out.println("Please enter how many players to play this game:");
        int numberOfPlayers = scanner.nextInt();

        Board board = new Board();
        Movement movement = new Movement();
        List<Player> players = new ArrayList<>();
        Banker banker = new Banker();
        GameHandler gameHandler = new GameAction(board);


        for (int i = 0; i < numberOfPlayers; i++) {
            System.out.println("Please enter player " + (i + 1) + "'s name");
            String playerName = scanner.next();
            Player player = new Player(playerName, 1000, 1);
            players.add(player);
        }

        int currentPlayerIndex = 0;
        int round = 0;

        while (true) {
            round++;
            System.out.println("Round " + round);

            for (int turn = 0; turn < numberOfPlayers; turn++) {
                Player currentPlayer = players.get(currentPlayerIndex);

                System.out.println("It's " + currentPlayer.getName() + "'s turn. Type 'R' to roll the die.");

                while(true) {
                    String input = scanner.next().toUpperCase();
                    if (input.equals("R")) {
                        movement.diceResult(currentPlayer);
                        break;
                    } else {
                        System.out.println("Invalid input. Type 'R' to roll the dice.");
                    }
                }

                int newPosition = currentPlayer.getPosition();
                String tileName = board.getGameBoard(newPosition);

                System.out.println(currentPlayer.getName() + " landed on " + tileName);
                currentPlayerIndex = (currentPlayerIndex + 1) % numberOfPlayers;
                System.out.println("You have £" + currentPlayer.getMoney() + " in your bank account ");

                if (newPosition == 4 || newPosition == 5 || newPosition == 9 || newPosition == 13 || newPosition == 15) {
                    if (board.canBuyProperty(newPosition)) {
                        gameHandler.landedOnProperty(currentPlayer);
                    }
                }

                if (newPosition == 7 || newPosition == 12 || newPosition == 14) {
                    gameHandler.landedOnTrainStation(currentPlayer);
                }

                if (newPosition == 1) {
                    banker.checkIfPlayerPassHome(currentPlayer);
                    System.out.println("Banker has given " + currentPlayer.getName() + " £200.");
                    currentPlayer.addMoney(200);
                    System.out.println("You now have £" + currentPlayer.getMoney() + " in your bank account ");
                }
            }

            if (GameEndings.checkRoundsReached(round, numberOfRounds)) {
                break;
            }
        }

        ArrayList<Player> playersArrayList = new ArrayList<>(players);
        GameEndings.determineWinner(playersArrayList);

        LOGGER.info("Application closing");
    }
}
